import os

import parsley
from jinja2 import Environment, FileSystemLoader

grammar_text = r"""
sections = section:first (ws section)*:rest -> [first] + rest
sectiontitle = '===' tablename '==='
section = sectiontitle ws table
tablestart = '{| '
tableend = '|}'
table = tablestartline ws rowstart ws tablemetaline:m ws rowstart ws '|' ws resulttable:r ws tableend -> (m, r)
resulttable = tablestartline ws (rowstart ws metarow ws)? rowstart ws rows:r ws tableend -> r
tablestartline = tablestart 'class="wikitable ' ('sortable' | 'collapsible') ws? '"' ('  border="1"')? ' width=' '"'? digit{1,3} '%' '"'? (' style="border: solid 2px white"')?
tablemetaline = '! colspan="5" style="background-color: ' letter+ ';" |' ' '? tablename:name -> name
tablename = <(letterOrDigit | '(' | ')' | '.' | ' ' | '&')+>
rowstart = '|-'
rowlinestart = '| '
rows = row:first (ws row)*:rest -> [first] + rest
row = ws resultrow:r ws rowstart -> r
metarow = '! width="' digit{1,3} '%"|Release Level !! width="' digit{1,3} '%"|Test Area !! width="' digit{1,3} '%"|Test Case !! width="' digit{1,3} '%"|i386 !! width="' digit{1,3} '%"|x86_64' (' !! width="' digit{1,3} '%"|EFI')?
resultrow = phaserow:phase ws testarearow:area ws testcaserow:testcase ws reportrow:report32 ws reportrow:report64 ws reportrow:reportefi -> {'phase':phase, 'area':area,'testcase':testcase, 'reports':{'i386':report32, 'x86_64':report64, 'EFI':reportefi}}
phaserow = rowlinestart phase
testarearow = rowlinestart testarea
testcaserow = rowlinestart testcasename
reportrow = rowlinestart (reports | ignorerowline)
testarea = <('Image Sanity' | 'Boot Methods' | 'Image Sanity' | 'Install Repository' | 'Package Set' | 'Partitioning' | 'User Interface' | 'Storage Devices' | 'Recovery' | 'Kickstart Delivery' | 'Upgrade system' | 'Memory Test')>
phase = <('Alpha/Final' | 'Alpha' | 'Beta' | 'Final' | 'Optional')>
ignorerowline = 'style="background:lightgrey;"|' -> [{'bug':'', 'user':'', 'result':'NA'}]
testcasename = '[[' ( 'QA:' (letterOrDigit | '-' | '_' | ':' | '|' | ' ' | '(' | ')' | '/' | '.')+):name ']]' -> ''.join(name)
reports = report:first (ws? report)*:rest-> [first] + rest
report = (reportstart reportdelim (namebugreport | namereport | nonamereport ):subreport reportend ) ref? -> subreport
namereport =  result:res reportdelim username:user -> {'result':res, 'user':user, 'bug':''}
namebugreport = result:res reportdelim username:user reportdelim (digit+):bugid -> {'result':res, 'user':user, 'bug':''.join(bugid)}
nonamereport = result:res -> {'result':res, 'user':'', 'bug':''}
ws = (' ' | '\n')*
username = < letter (letterOrDigit)+>
result = ('pass' | 'fail' | 'warn' | 'none'):x -> x
reportstart = '{{result'
reportdelim = '|'
reportend = '}}'
ref = (namedcontentref | namedref | unnamedcontentref )
namedref =  '<ref name=' (letterOrDigit+):refname '>' -> {'name':''.join(refname), 'text':''}
unnamedcontentref =  '<ref>' reftext:refcontent '</ref>' -> {'name':'', 'text':refcontent}
namedcontentref =  '<ref name=' (letterOrDigit+):refname '>' reftext:refcontent '</ref>'? -> {'name':''.join(refname), 'text':refcontent}
reftext = <(letterOrDigit | '!' | '@' | '#' | '$' | '%' | '^' | '&' | '*' | '(' | ')' | '-' | '_' | '"' | '\'' | ' ')+>
"""

def get_parser():
    return parsley.makeGrammar(grammar_text, {})

def get_files(directory, to_exclude = [], just_filename=False):
    dircontents = os.walk(directory)
    txtfiles = []
    for dircontent in dircontents:
        if len(dircontent[2]) == 0:
            continue
        for infile in dircontent[2]:
            filename = '%s/%s' % (dircontent[0], infile)

            if filename not in to_exclude:
                if just_filename:
                    txtfiles.append(os.path.basename(filename))
                else:
                    txtfiles.append(filename)
    return txtfiles

def get_file_contents(filename):
    f = open(filename, 'r')
    contents = f.read()
    f.close()
    return contents

def print_matrix(matrix):
    for table in matrix:
        print 'Table: %s' % table[0]
        for testcase in table[1]:
            print ''
            print '  testcase: %s' % testcase['testcase']
            print '    * phase: %s' % testcase['phase']
            print '    * reports:'
            print '       - i386: %s' % str(testcase['reports']['i386'])
            print '       - x86_64: %s' % str(testcase['reports']['x86_64'])
            print '       - EFI: %s' % str(testcase['reports']['x86_64'])
        print ''

def process_matrix(matrix):
    data = {}
    for table in matrix:
        data[table[0]] = []
        for testcase in table[1]:
            testinstance = {}
            testinstance['name'] = testcase['testcase']
            testinstance['phase'] = testcase['phase']

            reports = {}
            for arch in testcase['reports'].keys():
                archreports = []
                for report in testcase['reports'][arch]:
                    if report['result'] == 'none':
                        continue
                    else:
                        archreports.append(report['result'])

                reports[arch] = archreports
            testinstance['reports'] = reports

            data[table[0]].append(testinstance)
    return data

# assuming that matrices are structually identical across runs
def get_test_lists(matrix, name, base):
    for table in matrix:
        if table[0] not in base.keys():
            base[table[0]] = {}
        for testcase in table[1]:
            if testcase['testcase'] in base[table[0]].keys():
                testinstance = base[table[0]][testcase['testcase']]
                reports = testinstance['reports']
            else:
                testinstance = {}
                testinstance['name'] = testcase['testcase']
                testinstance['phase'] = testcase['phase']
                reports = {'i386':[], 'x86_64':[], 'EFI':[]}

            for arch in testcase['reports'].keys():
                for report in testcase['reports'][arch]:
                    if report['result'] == 'none':
                        continue
                    else:
                        reports[arch].append((name,report['result']))

            testinstance['reports'] = reports

            base[table[0]][testinstance['name']] = testinstance
    return base

def prettyprint_testlist(data):
    for tablename in data.keys():
        print tablename
        for testcasename in data[tablename].keys():
            testcase = data[tablename][testcasename]
            print '  testcase: %s' % testcase['name']
            print '    - i386:'
            print '       %s' % str(testcase['reports']['i386'])
            print '    - x86_64:'
            print '       %s' % str(testcase['reports']['x86_64'])
            print '    - EFI:'
            print '       %s' % str(testcase['reports']['EFI'])
            print ''
        print ''

def prettyprint_missingtestlist(data):
    for tablename in data.keys():
        print tablename
        for testcasename in data[tablename].keys():
            testcase = data[tablename][testcasename]
            print '  testcase: %s' % testcase['name']
            if len(testcase['reports']['i386']) == 0:
                print '    - i386:'
                print '       %s' % str(testcase['reports']['i386'])
            if len(testcase['reports']['x86_64']) == 0:
                print '    - x86_64:'
                print '       %s' % str(testcase['reports']['x86_64'])
            if len(testcase['reports']['EFI']) == 0:
                print '    - EFI:'
                print '       %s' % str(testcase['reports']['EFI'])
            print ''
        print ''

if __name__ == '__main__':
    basedir = '/home/tflink/code/wiki-results/data/'
    files = ['tc1','tc2','tc3']

    p = get_parser()

    data = {}
    for f in files:

        contents = get_file_contents('%s%s' % (basedir, f))

        matrix = p(contents.strip()).sections()

        data = get_test_lists(matrix, f, data)

    #prettyprint_testlist(data)
    prettyprint_missingtestlist(data)
